/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.vertx.java.test.junit;

import java.util.Comparator;

import org.junit.runner.Description;
import org.vertx.java.test.TestOrder;

/**
 * @author swilliams
 *
 */
public class OrderAnnotationComparator implements Comparator<Description> {

  @Override
  public int compare(Description one, Description two) {

    TestOrder order1 = one.getAnnotation(TestOrder.class);
    TestOrder order2 = two.getAnnotation(TestOrder.class);

    int order = 0;
    if (order1 != null && order2 != null) {
      order = order1.value() - order2.value();
    }
    else if (order1 != null && order2 == null) {
      return order1.value();
    }
    else if (order1 == null && order2 != null) {
      return order2.value();
    }

    if (order == 0) {
      order = one.getMethodName().compareTo(two.getMethodName());
    }

    return order;
  }

}
